/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the ViewMyCurrency extension.
 *
 * The Initial Developer of the Original Code is
 * Will Moffat.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//dump("// $Id: WILL_ViewMyCurrencyGui.js,v 1.2 2009/03/25 22:51:18 bogdan Exp $;\n");

/////////////////////
// ** GUI STUFF ** //
/////////////////////
VMC.popupConvertManager = null;

VMC.ViewMyCurrencyGui = function() {
  var self = this;
  var LABEL_POPUP_CONVERT='ViewMyCurrency: ';
  var gMouse = null; // set by _registerHandlers

  this._viewMyCurrency = null; //set by register func
  this._installStatus  = null;
  this._prefManager     = null; //set by registerViewMyCurrency

  this.registerViewMyCurrency      = registerViewMyCurrency;
  this.doContentMenuPopup          = doContentMenuPopup;
  this.doDisableTarget             = doDisableTarget; // only used programmatically (to view release notes without conversion)
  this.doHideSource                = doHideSource;
  this.doToggleConvert             = doToggleConvert;
  this.doUpdateStatus              = doUpdateStatus;
  this.doStatusbarpanelClicked     = doStatusbarpanelClicked;
  this.doContextMenu               = doContextMenu;
  this.doOpenOptionsDialog         = doOpenOptionsDialog;
  this.doGiveFeedback              = doGiveFeedback;
  this.doTest                      = doTest;
  this.doPopupConvert              = doPopupConvert;
  this.observe = observe;
  this.prefChangedCallBack         = prefChangedCallBack;


  this._initGuiBeforeInstall       = _initGuiBeforeInstall;
  this._initGuiAfterInstall        = _initGuiAfterInstall;
  this._confirmInstalled           = _confirmInstalled;
  this._considerDoingInstall       = _considerDoingInstall;
  this._wizardStartup              = _wizardStartup;
  this._normalStartup              = _normalStartup;
  this._upgradedStartup            = _upgradedStartup;
  this._runStartupWizard           = _runStartupWizard;
  this._showUpgradedPopup          = _showUpgradedPopup;
  this._showWelcomePopup           = _showWelcomePopup;
  this._registerHandlers           = _registerHandlers; //HACK: test that this new order works for clean install
  this.createTooltip               = createTooltip;  

  this._setStatusbarPanel          = _setStatusbarPanel;
  this.die   = die;

  this._initGuiBeforeInstall();
  
  function _initGuiBeforeInstall() {
    VMC.ge("VMC.statusbarpanel").addEventListener("mousedown",        function(e) { self.doStatusbarpanelClicked.call(self,e); }, false);
    this._setStatusbarPanel('OFF','','ViewMyCurrency v'+VMC.VERSION+' setup wizard is running');
  }

  function _initGuiAfterInstall() {
    if (!this._confirmInstalled()) {VMC.log('ERR','Tried to init whole gui before we were installed');}
    VMC.log('DBG','Registering full gui');
    VMC.ge("contentAreaContextMenu").addEventListener("popupshowing",  function(e) { self.doContentMenuPopup.call(self,e);      }, false);
    VMC.ge("VMC.statusbarpanel").contextMenu="VMC.statusIconPopup2";

    this._prefManager.registerWatcher(this, [VMC.PrefManager.keys.HIDE_IN_STATUSBAR], 'prefChangedCallBack');

    this.prefChangedCallBack(VMC.PrefManager.keys.HIDE_IN_STATUSBAR); // fake a pref change to hide/show based on preferences
    this.doUpdateStatus(); // draw the inital gui

  }

  // let the object that does all the work register with this GUI object
  function registerViewMyCurrency(viewMyCurrency) {
    this._viewMyCurrency = viewMyCurrency;
    this._viewMyCurrency.updateGuiCallback = function(status) { self.doUpdateStatus(status) };    

    this._prefManager=viewMyCurrency.prefManager;

    var prefState=this._prefManager.determineStatusOfExistingPrefs();
    VMC.log('DBG','State of preferences: '+prefState);
    switch(prefState) {

    case VMC.PrefManager.stateVirgin:                   this._wizardStartup();     break; // cool. I got a new user of VMC!

    case VMC.PrefManager.stateCannotDetermineVersion:   
    case VMC.PrefManager.stateIncompatibleOlderVersion: 
    case VMC.PrefManager.stateCorrupt:                  this._wizardStartup(true); break; // old user, warn about deleting prefs

    case VMC.PrefManager.stateValidPrefsAndVmcUpgraded: this._upgradedStartup();   break; // prefs are OK but VMC has been updated. 
    case VMC.PrefManager.stateValidPrefsPresent:        this._normalStartup();     break; // no wizard required

    default : VMC.log('ERR','Unknown pref state '+prefState);
    }
  }

  function _upgradedStartup() {
    this._normalStartup(); //do the normal stuff
    this._prefManager.setCurrentVersion(); // write the new version into the prefs
    this._showUpgradedPopup(); // inform the user that this is a new version
  }

  function _normalStartup(wasWizardStartup) {
    //normal startup
    this._installStatus="ALREADY_INSTALLED";
    try {
      this._viewMyCurrency._initAfterInstall(wasWizardStartup);
      this._registerHandlers();
    } catch (e) {
      var message='Please let me know at converter@moongate.ro   The error was:\n'+e;
      if (wasWizardStartup) {
	VMC.log('ERR','Oh dear! You just did a clean install but ViewMyCurrency prefs seems to be corrupt.\n'+message);
      } else {
	VMC.log('WARN','It looks like your ViewMyCurrency preferences are corrupt.\n'+message);
	this._wizardStartup(true); // try install again, warn about deleting prefs
      }
    }

    this._viewMyCurrency.exchangeRateCache.webUpdater.addObserver(this);

    this._initGuiAfterInstall();
  }


  function _registerHandlers() {
    //get the tabbrowser  OR get the mini-browser if we are in test mode (true = don't complain on failure)
    var appcontent = VMC.ge("appcontent",true) || VMC.ge('VMC.miniBrowser',true);
    if (!appcontent) {
      VMC.alert('Could not find appcontent - is ViewMyCurrency running in Firefox?');
      return;
    }
    gMouse = new VMC.MouseHandler(this._viewMyCurrency,createTooltip);
    appcontent.addEventListener("mousemove", gMouse.onMousemove, false);
    appcontent.addEventListener("DOMContentLoaded", function(e) { self._viewMyCurrency._onDOMContentLoaded.call(self._viewMyCurrency,e);  }, false);
    //WILL: TODO: add and remove this listeners as required rather than just prematurely returning if their function isn't needed
   
  }

  function _runStartupWizard(deletePrefs) {
    var retVal = {};
    window.openDialog( 'chrome://viewmycurrency/content/WILL_StartUpDialog.xul','VMC.StartUpDialog','chrome,resizable,modal',
		       VMC.currencyDB.getCompleteList(),this._prefManager.getDefaultSourceListPref(),deletePrefs,retVal );
    if (retVal['newTargetCurrencyList']) {
      var newTargetCurrencyList=retVal['newTargetCurrencyList'];
      this._prefManager.storeInitialPrefs(newTargetCurrencyList,deletePrefs);

      return true; // wizard compelted OK
    } else {
      return false; // the user clicked cancel
    }
  }

  function _showUpgradedPopup() {
    VMC.updateAndAddHandlerToLinks();
    var messageLabel = VMC.ge('VMC-upgraded-message');
    messageLabel.value="Upgraded to "+VMC.VERSION;

    var statusbarpanel = VMC.ge('VMC.statusbarpanel');
    var myPopup = VMC.ge('VMC-upgraded-popup');
    myPopup.showPopup(statusbarpanel,-1,-1,'popup','topright','bottomright'); 
  }

  function _showWelcomePopup() {
    // Display the welcome popup
    var statusbarpanel = VMC.ge('VMC.statusbarpanel');
    var myPopup = VMC.ge('VMC-installed-popup');
    myPopup.showPopup(statusbarpanel,-1,-1,'popup','topright','bottomright');
  }

  //  WILL: TODO: I used to delay wizard, it is necessary? this._scheduleCall('delayedInstall',3*1000); 
  function _wizardStartup(deletePrefs) {
    var configOK = this._runStartupWizard(deletePrefs);
    //this._prefManager.doFirstTimeSetup();

    if (configOK) {
      this._normalStartup(true); //do a normal startup but run the webupdate immediately
      this._showWelcomePopup();  // highlight the fact that we live on the statusbar
    } else {
      this._installStatus="DONT_INSTALL";
    }
  }

  function die() {
    if (!this._confirmInstalled()) { return; }
    this._viewMyCurrency.die();
    this._viewMyCurrency.exchangeRateCache.webUpdater.removeObserver(this);
    //WILL: TODO: currently the VMC.PrefManager.unregister()  kills all watchers, so we just do that in _viewMyCurrency.die()
  }

  function doToggleConvert() {
    this._viewMyCurrency.changeToNextTargetCurrency();
  }

  function doDisableTarget() { //used by FeedBack Wizard
    this._viewMyCurrency.disableConversion(null); 
    
    // give visual cue that we went automatically to OFF. WILL: TODO: improve this. Not seen in child windows.
    setTimeout(function() { self._setStatusbarPanel(null, 'color:black; font-weight:bold;', 'Automatically switched to OFF') }, 300 );
  }
  
  function doHideSource() {
    this._viewMyCurrency.toggleHideOriginal();
  }

  
  function _confirmInstalled() {
    if (!this._installStatus) { return false; }
    return (this._installStatus == 'ALREADY_INSTALLED');
  }

  //WILL: TODO: I don't think I need all these states if I run the wizard without delay
  function _considerDoingInstall() {
    switch(this._installStatus) {
    case "ALREADY_INSTALLED": VMC.log('ERR','We are already installed!'); break;
    case "TO_INSTALL":        return; // install should happen soon (by timeout)
    case "DONT_INSTALL":     //the user cancelled that last setup wizard, but now he seems to want our functionality
      this.registerViewMyCurrency(this._viewMyCurrency); // fake the inital startup flow of control to run the wizard again.
      break;
    default:
      VMC.log('ERR','unknown install status :'+this._installStatus);
    }
  }


  /////////////////////////////
  function doStatusbarpanelClicked (event) {
    /////////////////////////////
    if (! this._confirmInstalled() ) { this._considerDoingInstall(); return; }
    
    if (event.button !== 0) { return; } //WILL: This wasn't a left-click so do nothing here (right-click brings up menu). TODO: test on mac and unix

    this.doToggleConvert();

  }

  // label is optional
  function _setStatusbarPanel(label,style,tooltiptext) {
    var statusbarpanel = VMC.ge("VMC.statusbarpanel");
    if (label)       { statusbarpanel.label=label }
    if (tooltiptext) { statusbarpanel.setAttribute('tooltiptext',tooltiptext) }
    if (style)       { statusbarpanel.setAttribute('style','width:6em; '+style) }  //WILL: TODO: use style selectors
  }

  ////////////////////////
  function doUpdateStatus (state) {
    //////////////////////
    //WILL: TODO: maybe we should seperate this into updateStatusBar and updateMenus?
    switch(state) {
    case "START" :       this._setStatusbarPanel(null,'color:green; font-weight:bold', null);  return;
    case "END_CHANGES" : this._setStatusbarPanel(null,'font-weight:bold', null);               return;
    case "END_NO_CHNG" : this._setStatusbarPanel(null,' ', null);                              return;
    case "OFF"         : this._setStatusbarPanel('OFF','color:green',null);                    return; // we are in progress of reverting page
    case "AVOID"       : this._setStatusbarPanel(null,'color:grey','Not converting (this page is blacklisted)' ); return; // we are in progress of reverting page
    default : // do a complete gui update
    }

    var hide=this._viewMyCurrency.hideOriginalCurrency;
    VMC.ge('VMC.hideOriginal' ).setAttribute('checked',hide);
    VMC.ge('VMC.hideOriginal2').setAttribute('checked',hide);


    var target    =this._viewMyCurrency.getTargetCurrency();
    var nextTarget=this._viewMyCurrency.getNextTargetCurrency();

    var menuLabel;
    if (nextTarget) { menuLabel='Convert to '+nextTarget.code;  }
    else            { menuLabel='Convert OFF';                  }
    VMC.ge('VMC.cmd_toggleConvert').setAttribute('label',menuLabel);

    if (!target) {
      this._setStatusbarPanel('OFF','color:black','Not converting currencies. Click to change');
    } else {
      this._setStatusbarPanel(target.code+' '+target.getHtml(), null, 'Converting to '+target.description ); //don't change style
    }
  }

  // used to observe the WebUpdater. //Note: this only deals with gui stuff. VMC is responsible for updating browsers
  function observe(status) {
    var statusbarpanel = VMC.ge("VMC.statusbarpanel");    
    var style;
    var tooltiptext;

    switch(status) {
    case 'START'        : this._setStatusbarPanel(null, 'color:blue;', 'Getting the latest exchange rates from web'); break;
    case 'END_ERRORS'   : this._setStatusbarPanel(null, 'color:red;',  'Failed to download all exchange rates');      break;
    case 'END_OK'       : this._setStatusbarPanel(null, '',            'Downloaded exchange rates OK');               break;
    case 'RATE_ERROR'   : this._setStatusbarPanel(null, 'color:darkred;','Failed to download exchange rate. (Continuing with the rest)'); break;
    case 'RATE_UPDATED' : break;      // do nothing
    default             : VMC.log('ERR','got unknown webupdate status ['+status+']');
    }
  }

  function getSelectedText() {
    if (this.getBrowserSelection)         { return getBrowserSelection(100); } // Firefox 2
    // else Firefox 1.5
    var sel=nsContextMenu.prototype.searchSelected(100); // can't use gContextMenu because there isn't a menu showing
    if (sel) { return sel.toString() }
    else { return null; }

    //if (gContextMenu.searchSelected) { return gContextMenu.searchSelected(100); } //Firefox 1.5
    //VMC.log('ERR','Could not find selected text function');
    //return undefined;
  }

  // the user right-clicked inside the broswer. Now we decided if we should we add our menuitem
  function doContentMenuPopup(event) {
    if (!this._confirmInstalled()) { VMC.log('ERR','Right click menu should not be here until wizard is finished'); }

    var menuitem;
    var showPopupConvert;
    var seltext;

    var sel = getSelectedText();
    if (sel) {
      seltext = sel.toString();
      showPopupConvert=true;
    }
    menuitem = VMC.ge('VMC.contentMenuitemP');
    if (showPopupConvert) {
      menuitem.label=LABEL_POPUP_CONVERT+' "'+VMC.shortenStr(seltext,16)+'"';
      menuitem.hidden=false;
    } else {
      menuitem.hidden=true;
    }

    //WILL: TODO: consider separator and order of menuitems
  }


  // helper func
  function _getSelectedTextRange() {
    var focusedWindow = document.commandDispatcher.focusedWindow;
    var selectionObj = focusedWindow.getSelection();      
    if (!selectionObj.rangeCount)  { return null; }
    if (selectionObj.rangeCount>1) { VMC.alert('warning selection rangeCount is '+selectionObj.rangeCount+' only looking at first item'); }
    return selectionObj.getRangeAt(0);
  }


  // the user click on our menuitem in either the right-click context menu of the browser, our statusbar, our tools menu or pressed keyboard shortcut
  function doPopupConvert() {
    // run the popup converter
    var range=_getSelectedTextRange();
    var focus = document.commandDispatcher.focusedWindow; // so we can restore focus when we hide the toolbar
    VMC.popupConvertManager = new VMC.PopupConvertToolbarManager(this._viewMyCurrency,range,focus);
  }
    

  function doContextMenu(menuitem) {
    
    if (menuitem.label.substr(0,LABEL_POPUP_CONVERT.length) != LABEL_POPUP_CONVERT) {
      VMC.log('ERR','Unknown menuitem '+menuitem.label);
      return;
    }

    this.doPopupConvert();

  }
  
  function doOpenOptionsDialog() {
    if (! this._confirmInstalled() ) { this._considerDoingInstall(); return; }
    window.openDialog('chrome://viewmycurrency/content/WILL_OptionsDialog.xul','VMC.OptionsDialog','chrome,resizable,modal');
    // This modifies preferences. Changes are detected and processed by ViewMyCurrency
  }
 
  function doGiveFeedback() {
    var url='';
    var selectedText='';
    try {
      url = gBrowser.getBrowserAtIndex(gBrowser.mTabContainer.selectedIndex).contentDocument.URL;
      selectedText = getSelectedText();
    } catch (e) { VMC.log('WARN','could not find current URL or selected text '+e) }
    
    window.openDialog('chrome://viewmycurrency/content/WILL_GiveFeedback.xul','VMC.GiveFeedback','chrome,resizable',
		      url, selectedText);
  }

  
  function prefChangedCallBack(prefName) {
    VMC.log('DBG','GUI pref changed: '+prefName);

    var keys=VMC.PrefManager.keys;

    switch(prefName) {
    case keys.HIDE_IN_STATUSBAR   : VMC.ge('VMC.statusbarpanel').hidden=this._prefManager.getHideInStatusbar(); break;
    default                       : VMC.log('ERR','Saw unexpected pref '+prefName+' change'); return;
    }
  }

  var gTooltipNode = document.documentElement; //VMC.ge('appcontent'); //BUG: tooltips don't always appear if you add them to appcontent
  var mouseoverTooltip        = VMC.ge('VMC-mouseover-tooltip');
  var mouseoverConvertedLabel = VMC.ge('VMC-mouseoverConverted-label');
  var mouseoverOriginalLabel  =  VMC.ge('VMC-mouseoverOriginal-label');


  function createTooltip() {
    var text = VMC.HtmlConversion.findTextUnderMouse(gMouse.parent,gMouse.offset);
    if (text) {
      var vmc = self._viewMyCurrency;
      var r = VMC.NumberConversion.prototype.currencyConvertStrForTooltip(text, vmc._targetCurrency, vmc.conversionProfileTable._activeProfileEntry.profile, vmc.getRateFunc); //HACK: horrible OOP + HACK: need to set active profile when page loads
    }
    if (!r) {
      mouseoverTooltip.hidePopup();
    } else {
      var convertedText = r.convertedText;
      var originalText  = r.originalText;
	mouseoverConvertedLabel.value = convertedText;
	mouseoverOriginalLabel.value  = originalText;
	//W: document.popupNode = null; mouseoverTooltip.hidePopup(); //BUG: sometimes it appears in wrong location
	//W: document.popupNode = gTooltipNode; //BUG: this should not be necessary, not sure if it makes a difference
	mouseoverTooltip.showPopup(gTooltipNode,gMouse.x,gMouse.y,'tooltip',null,null);  //BUG: I'm using screen coordinates, so why do I have to specify element?
	//W: vmc.hack('Tooltip: x='+gMouse.x+'\ty='+gMouse.y+'\t\t'+r.convertedText+'  '+r.originalText); //BUG: sometimes this tooltip never appears
    }
  }

  function doTest() {
    this.doDisableTarget();
    //this.doGiveFeedback();
    //this._showWelcomePopup();
  }


}; // end of ViewMyCurrencyGui

VMC.MouseHandler = function(vmc,onMouseMovementFinished) {
  var self = this;
  var MOUSE_CHECK_INTERVAL = 50;
  this.parent = null;
  this.offset = null;
  this.x = null;
  this.y = null;
  this.moved = false;
  this.mouseTimer = null;

  this.onMousemove = function onMousemove(event) {   // BUG: this basically implmements missing event mouseMovementFinished
    if (!vmc.tooltipMode) { return; }
    //dump('.'); // Mouse move event
    self.x = event.screenX;
    self.y = event.screenY;
    self.parent = event.rangeParent;
    self.offset = event.rangeOffset;
    if (!self.mouseTimer) {
      self.moved = false; //reset the movement flag
      self.mouseTimer = window.setTimeout(self.mousePoll,MOUSE_CHECK_INTERVAL);
    } else {
      self.moved = true;
    }
  };
  
  this.mousePoll = function() {
    if (self.moved) {
      //dump('-');  //mouse has moved recently
      self.moved = false; //reset the movement flag
      self.mouseTimer = window.setTimeout(arguments.callee,MOUSE_CHECK_INTERVAL); //call me again
    } else {
      //dump('X');  //mouse has stopped moving
      self.mouseTimer = null;
      onMouseMovementFinished()
    }
  };
  
}; // end of MouseHandler
